@@ -1,4 +1,5 @@
-# (C) Copyright 2014-2016 Hewlett Packard Enterprise Development Company LP
+# (C) Copyright 2014-2017 Hewlett Packard Enterprise Development LP
+# Copyright 2017 FUJITSU LIMITED
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,12 +19,12 @@ import json
 import numbers
 import time
 
+from keystoneauth1 import exceptions as k_exc
+from osc_lib import exceptions as osc_exc
 
 from monascaclient.common import utils
-import monascaclient.exc as exc
 
 from oslo_serialization import jsonutils
-from six.moves import xrange
 
 # Alarm valid types
 severity_types = ['LOW', 'MEDIUM', 'HIGH', 'CRITICAL']
@@ -65,8 +66,7 @@ notification_types = ['EMAIL', 'WEBHOOK', 'PAGERDUTY']
            help='Metric timestamp in milliseconds. Default: current timestamp.')
 @utils.arg('--project-id', metavar='<CROSS_PROJECT_ID>',
            help='The Project ID to create metric on behalf of. '
-           'Requires monitoring-delegate role in keystone.',
-           action='append')
+           'Requires monitoring-delegate role in keystone.')
 @utils.arg('value', metavar='<METRIC_VALUE>',
            type=float,
            help='Metric value.')
@@ -84,10 +84,8 @@ def do_metric_create(mc, args):
         fields['tenant_id'] = args.project_id
     try:
         mc.metrics.create(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print('Successfully created metric')
 
@@ -97,14 +95,10 @@ def do_metric_create(mc, args):
            help='The raw JSON body in single quotes. See api doc.')
 def do_metric_create_raw(mc, args):
     '''Create metric from raw json body.'''
-    fields = {}
-    fields['jsonbody'] = args.jsonbody
     try:
-        mc.metrics.create(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+        mc.metrics.create(**args.jsonbody)
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print('Successfully created metric')
 
@@ -137,17 +131,14 @@ def do_metric_name_list(mc, args):
 
     try:
         metric_names = mc.metrics.list_names(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
-
-    if args.json:
-        print(utils.json_formatter(metric_names))
-        return
-
-    if isinstance(metric_names, list):
-        utils.print_list(metric_names, ['Name'], formatters={'Name': lambda x: x['name']})
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
+    else:
+        if args.json:
+            print(utils.json_formatter(metric_names))
+            return
+        if isinstance(metric_names, list):
+            utils.print_list(metric_names, ['Name'], formatters={'Name': lambda x: x['name']})
 
 
 @utils.arg('--name', metavar='<METRIC_NAME>',
@@ -191,10 +182,8 @@ def do_metric_list(mc, args):
 
     try:
         metric = mc.metrics.list(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(metric))
@@ -217,6 +206,83 @@ def do_metric_list(mc, args):
                 formatters=formatters)
 
 
+@utils.arg('--metric-name', metavar='<METRIC_NAME>',
+           help='Name of the metric to report dimension name list.',
+           action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum '
+                'limit.')
+@utils.arg('--tenant-id', metavar='<TENANT_ID>',
+           help="Retrieve data for the specified tenant/project id instead of "
+                "the tenant/project from the user's Keystone credentials.")
+def do_dimension_name_list(mc, args):
+    '''List names of metric dimensions.'''
+    fields = {}
+    if args.metric_name:
+        fields['metric_name'] = args.metric_name
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.tenant_id:
+        fields['tenant_id'] = args.tenant_id
+
+    try:
+        dimension_names = mc.metrics.list_dimension_names(**fields)
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
+
+    if args.json:
+        print(utils.json_formatter(dimension_names))
+        return
+
+    if isinstance(dimension_names, list):
+        utils.print_list(dimension_names, ['Dimension Names'], formatters={
+            'Dimension Names': lambda x: x['dimension_name']})
+
+
+@utils.arg('dimension_name', metavar='<DIMENSION_NAME>',
+           help='Name of the dimension to list dimension values.')
+@utils.arg('--metric-name', metavar='<METRIC_NAME>',
+           help='Name of the metric to report dimension value list.',
+           action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum '
+                'limit.')
+@utils.arg('--tenant-id', metavar='<TENANT_ID>',
+           help="Retrieve data for the specified tenant/project id instead of "
+                "the tenant/project from the user's Keystone credentials.")
+def do_dimension_value_list(mc, args):
+    '''List names of metric dimensions.'''
+    fields = {}
+    fields['dimension_name'] = args.dimension_name
+    if args.metric_name:
+        fields['metric_name'] = args.metric_name
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.tenant_id:
+        fields['tenant_id'] = args.tenant_id
+
+    try:
+        dimension_values = mc.metrics.list_dimension_values(**fields)
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
+
+    if args.json:
+        print(utils.json_formatter(dimension_values))
+        return
+
+    if isinstance(dimension_values, list):
+        utils.print_list(dimension_values, ['Dimension Values'], formatters={
+            'Dimension Values': lambda x: x['dimension_value']})
+
+
 def format_measure_timestamp(measurements):
     # returns newline separated times for the timestamp column
     return '\n'.join([str(m[0]) for m in measurements])
@@ -349,10 +415,8 @@ def do_measurement_list(mc, args):
 
     try:
         metric = mc.metrics.list_measurements(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(metric))
@@ -414,10 +478,10 @@ def do_metric_statistics(mc, args):
     statlist = args.statistics.split(',')
     for stat in statlist:
         if stat.upper() not in statistic_types:
-            errmsg = 'Invalid type, not one of [' + \
-                ', '.join(statistic_types) + ']'
-            print(errmsg)
-            return
+            errmsg = ('Invalid type, not one of [' +
+                      ', '.join(statistic_types) + ']')
+            raise osc_exc.CommandError(errmsg)
+
     fields = {}
     fields['name'] = args.name
     if args.dimensions:
@@ -442,10 +506,8 @@ def do_metric_statistics(mc, args):
 
     try:
         metric = mc.metrics.list_statistics(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(metric))
@@ -507,11 +569,7 @@ def _validate_notification_period(period, notification_type):
            help='A period for the notification method. Can only be non zero with webhooks')
 def do_notification_create(mc, args):
     '''Create notification.'''
-    if args.type.upper() not in notification_types:
-        errmsg = 'Invalid type, not one of [' + \
-            ', '.join(notification_types) + ']'
-        print(errmsg)
-        return
+
     fields = {}
     fields['name'] = args.name
     fields['type'] = args.type
@@ -522,26 +580,22 @@ def do_notification_create(mc, args):
         fields['period'] = args.period
     try:
         notification = mc.notifications.create(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(notification, indent=2))
 
 
 @utils.arg('id', metavar='<NOTIFICATION_ID>',
-           help='The ID of the notification. If not specified returns all.')
+           help='The ID of the notification.')
 def do_notification_show(mc, args):
     '''Describe the notification.'''
     fields = {}
     fields['notification_id'] = args.id
     try:
         notification = mc.notifications.get(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(notification))
@@ -589,9 +643,9 @@ def do_notification_list(mc, args):
 
     try:
         notification = mc.notifications.list(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
+    except osc_exc.ClientException as he:
+        raise osc_exc.CommandError(
+            'ClientException code=%s message=%s' %
             (he.code, he.message))
     else:
         if args.json:
@@ -625,10 +679,8 @@ def do_notification_delete(mc, args):
     fields['notification_id'] = args.id
     try:
         mc.notifications.delete(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print('Successfully deleted notification')
 
@@ -648,11 +700,7 @@ def do_notification_update(mc, args):
     fields = {}
     fields['notification_id'] = args.id
     fields['name'] = args.name
-    if args.type.upper() not in notification_types:
-        errmsg = 'Invalid type, not one of [' + \
-                 ', '.join(state_types) + ']'
-        print(errmsg)
-        return
+
     fields['type'] = args.type
     fields['address'] = args.address
     if not _validate_notification_period(args.period, args.type.upper()):
@@ -660,18 +708,50 @@ def do_notification_update(mc, args):
     fields['period'] = args.period
     try:
         notification = mc.notifications.update(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
+    else:
+        print(jsonutils.dumps(notification, indent=2))
+
+
+@utils.arg('id', metavar='<NOTIFICATION_ID>',
+           help='The ID of the notification.')
+@utils.arg('--name', metavar='<NOTIFICATION_NAME>',
+           help='Name of the notification.')
+@utils.arg('--type', metavar='<TYPE>',
+           help='The notification type. Type must be either EMAIL, WEBHOOK, or PAGERDUTY.')
+@utils.arg('--address', metavar='<ADDRESS>',
+           help='A valid EMAIL Address, URL, or SERVICE KEY.')
+@utils.arg('--period', metavar='<PERIOD>', type=int,
+           help='A period for the notification method. Can only be non zero with webhooks')
+def do_notification_patch(mc, args):
+    '''Patch notification.'''
+    fields = {}
+    fields['notification_id'] = args.id
+    if args.name:
+        fields['name'] = args.name
+
+    if args.type:
+        fields['type'] = args.type
+    if args.address:
+        fields['address'] = args.address
+    if args.period or args.period == 0:
+        if args.type and not _validate_notification_period(
+                args.period, args.type.upper()):
+            return
+        fields['period'] = args.period
+    try:
+        notification = mc.notifications.patch(**fields)
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(notification, indent=2))
 
 
 def _validate_severity(severity):
     if severity.upper() not in severity_types:
-        errmsg = 'Invalid severity, not one of [' + \
-            ', '.join(severity_types) + ']'
+        errmsg = ('Invalid severity, not one of [' +
+                  ', '.join(severity_types) + ']')
         print(errmsg)
         return False
     return True
@@ -724,10 +804,8 @@ def do_alarm_definition_create(mc, args):
         fields['match_by'] = args.match_by.split(',')
     try:
         alarm = mc.alarm_definitions.create(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(alarm, indent=2))
 
@@ -740,10 +818,8 @@ def do_alarm_definition_show(mc, args):
     fields['alarm_id'] = args.id
     try:
         alarm = mc.alarm_definitions.get(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(alarm))
@@ -815,10 +891,8 @@ def do_alarm_definition_list(mc, args):
         fields['offset'] = args.offset
     try:
         alarm = mc.alarm_definitions.list(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(alarm))
@@ -849,10 +923,8 @@ def do_alarm_definition_delete(mc, args):
     fields['alarm_id'] = args.id
     try:
         mc.alarm_definitions.delete(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print('Successfully deleted alarm definition')
 
@@ -895,8 +967,8 @@ def do_alarm_definition_update(mc, args):
     fields['ok_actions'] = _arg_split_patch_update(args.ok_actions)
     fields['undetermined_actions'] = _arg_split_patch_update(args.undetermined_actions)
     if args.actions_enabled not in enabled_types:
-        errmsg = 'Invalid value, not one of [' + \
-            ', '.join(enabled_types) + ']'
+        errmsg = ('Invalid value, not one of [' +
+                  ', '.join(enabled_types) + ']')
         print(errmsg)
         return
     fields['actions_enabled'] = args.actions_enabled in ['true', 'True']
@@ -906,10 +978,8 @@ def do_alarm_definition_update(mc, args):
     fields['severity'] = args.severity
     try:
         alarm = mc.alarm_definitions.update(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(alarm, indent=2))
 
@@ -956,8 +1026,8 @@ def do_alarm_definition_patch(mc, args):
         fields['undetermined_actions'] = _arg_split_patch_update(args.undetermined_actions, patch=True)
     if args.actions_enabled:
         if args.actions_enabled not in enabled_types:
-            errmsg = 'Invalid value, not one of [' + \
-                ', '.join(enabled_types) + ']'
+            errmsg = ('Invalid value, not one of [' +
+                      ', '.join(enabled_types) + ']')
             print(errmsg)
             return
         fields['actions_enabled'] = args.actions_enabled in ['true', 'True']
@@ -967,10 +1037,8 @@ def do_alarm_definition_patch(mc, args):
         fields['severity'] = args.severity
     try:
         alarm = mc.alarm_definitions.patch(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(alarm, indent=2))
 
@@ -979,12 +1047,13 @@ def do_alarm_definition_patch(mc, args):
            help='The ID of the alarm definition.')
 @utils.arg('--metric-name', metavar='<METRIC_NAME>',
            help='Name of the metric.')
-@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
-           help='key value pair used to specify a metric dimension. '
-           'This can be specified multiple times, or once with parameters '
-           'separated by a comma. '
-           'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
-           'that confuse the CLI parser.',
+@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2,KEY3=VALUE2...>',
+           help='key value pair used to specify a metric dimension or '
+                'just key to select all values of that dimension.'
+                'This can be specified multiple times, or once with parameters '
+                'separated by a comma. '
+                'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
+                'that confuse the CLI parser.',
            action='append')
 @utils.arg('--state', metavar='<ALARM_STATE>',
            help='ALARM_STATE is one of [UNDETERMINED, OK, ALARM].')
@@ -1017,8 +1086,8 @@ def do_alarm_list(mc, args):
         fields['metric_dimensions'] = utils.format_dimensions_query(args.metric_dimensions)
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1050,10 +1119,8 @@ def do_alarm_list(mc, args):
         fields['sort_by'] = args.sort_by
     try:
         alarm = mc.alarms.list(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(alarm))
@@ -1093,10 +1160,8 @@ def do_alarm_show(mc, args):
     fields['alarm_id'] = args.id
     try:
         alarm = mc.alarms.get(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(alarm))
@@ -1125,8 +1190,8 @@ def do_alarm_update(mc, args):
     fields = {}
     fields['alarm_id'] = args.id
     if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
     fields['state'] = args.state
@@ -1134,10 +1199,8 @@ def do_alarm_update(mc, args):
     fields['link'] = args.link
     try:
         alarm = mc.alarms.update(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(alarm, indent=2))
 
@@ -1156,8 +1219,8 @@ def do_alarm_patch(mc, args):
     fields['alarm_id'] = args.id
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1167,10 +1230,8 @@ def do_alarm_patch(mc, args):
         fields['link'] = args.link
     try:
         alarm = mc.alarms.patch(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print(jsonutils.dumps(alarm, indent=2))
 
@@ -1183,10 +1244,8 @@ def do_alarm_delete(mc, args):
     fields['alarm_id'] = args.id
     try:
         mc.alarms.delete(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         print('Successfully deleted alarm')
 
@@ -1222,17 +1281,20 @@ def output_alarm_history(args, alarm_history):
            help='The ID of the alarm definition.')
 @utils.arg('--metric-name', metavar='<METRIC_NAME>',
            help='Name of the metric.')
-@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
-           help='key value pair used to specify a metric dimension. '
-           'This can be specified multiple times, or once with parameters '
-           'separated by a comma. '
-           'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
-           'that confuse the CLI parser.',
+@utils.arg('--metric-dimensions', metavar='<KEY1=VALUE1,KEY2,KEY3=VALUE2...>',
+           help='key value pair used to specify a metric dimension or '
+                'just key to select all values of that dimension.'
+                'This can be specified multiple times, or once with parameters '
+                'separated by a comma. '
+                'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
+                'that confuse the CLI parser.',
            action='append')
 @utils.arg('--state', metavar='<ALARM_STATE>',
            help='ALARM_STATE is one of [UNDETERMINED, OK, ALARM].')
 @utils.arg('--severity', metavar='<SEVERITY>',
            help='Severity is one of ["LOW", "MEDIUM", "HIGH", "CRITICAL"].')
+@utils.arg('--state-updated-start-time', metavar='<UTC_STATE_UPDATED_START>',
+           help='Return all alarms whose state was updated on or after the time specified.')
 @utils.arg('--lifecycle-state', metavar='<LIFECYCLE_STATE>',
            help='The lifecycle state of the alarm.')
 @utils.arg('--link', metavar='<LINK>',
@@ -1253,11 +1315,11 @@ def do_alarm_count(mc, args):
     if args.metric_name:
         fields['metric_name'] = args.metric_name
     if args.metric_dimensions:
-        fields['metric_dimensions'] = utils.format_parameters(args.metric_dimensions)
+        fields['metric_dimensions'] = utils.format_dimensions_query(args.metric_dimensions)
     if args.state:
         if args.state.upper() not in state_types:
-            errmsg = 'Invalid state, not one of [' + \
-                ', '.join(state_types) + ']'
+            errmsg = ('Invalid state, not one of [' +
+                      ', '.join(state_types) + ']')
             print(errmsg)
             return
         fields['state'] = args.state
@@ -1265,6 +1327,8 @@ def do_alarm_count(mc, args):
         if not _validate_severity(args.severity):
             return
         fields['severity'] = args.severity
+    if args.state_updated_start_time:
+        fields['state_updated_start_time'] = args.state_updated_start_time
     if args.lifecycle_state:
         fields['lifecycle_state'] = args.lifecycle_state
     if args.link:
@@ -1272,8 +1336,8 @@ def do_alarm_count(mc, args):
     if args.group_by:
         group_by = args.group_by.split(',')
         if not set(group_by).issubset(set(group_by_types)):
-            errmsg = 'Invalid group-by, one or more values not in [' + \
-                ','.join(group_by_types) + ']'
+            errmsg = ('Invalid group-by, one or more values not in [' +
+                      ','.join(group_by_types) + ']')
             print(errmsg)
             return
         fields['group_by'] = args.group_by
@@ -1283,17 +1347,15 @@ def do_alarm_count(mc, args):
         fields['offset'] = args.offset
     try:
         counts = mc.alarms.count(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         if args.json:
             print(utils.json_formatter(counts))
             return
         cols = counts['columns']
 
-        utils.print_list(counts['counts'], [i for i in xrange(len(cols))],
+        utils.print_list(counts['counts'], [i for i in range(len(cols))],
                          field_labels=cols)
 
 
@@ -1313,10 +1375,8 @@ def do_alarm_history(mc, args):
         fields['offset'] = args.offset
     try:
         alarm = mc.alarms.history(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         output_alarm_history(args, alarm)
 
@@ -1352,14 +1412,29 @@ def do_alarm_history_list(mc, args):
         fields['offset'] = args.offset
     try:
         alarm = mc.alarms.history_list(**fields)
-    except exc.HTTPException as he:
-        raise exc.CommandError(
-            'HTTPException code=%s message=%s' %
-            (he.code, he.message))
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
     else:
         output_alarm_history(args, alarm)
 
 
+def do_notification_type_list(mc, args):
+    '''List notification types supported by monasca.'''
+
+    try:
+        notification_types = mc.notificationtypes.list()
+    except (osc_exc.ClientException, k_exc.HttpError) as he:
+        raise osc_exc.CommandError('%s\n%s' % (he.message, he.details))
+    else:
+        if args.json:
+            print(utils.json_formatter(notification_types))
+            return
+        else:
+            formatters = {'types': lambda x: x["type"]}
+            # utils.print_list(notification_types['types'], ["types"], formatters=formatters)
+            utils.print_list(notification_types, ["types"], formatters=formatters)
+
+
 def _translate_starttime(args):
     if args.starttime[0] == '-':
         deltaT = time.time() + (int(args.starttime) * 60)
